// This is a copy of file from nBMSL. Seems to be the cleaned up version.
// although testing of typecheck etc has not been done.

#include <unistd.h>
#include <fstream>
#include "ParserUtil.h"
#include "CondFac.h"

#define MAX_DEBUG_LEVEL 2
#define CPP_PROG_NAME "CPP" // Environment variable to specify location of cpp
#define DEFAULT_CPP_PROG_NAME "cpp"

#ifdef DEBUG_LEXER
#ifdef LEXER_OUT
ofstream lexOutStr(LEXER_OUT);
ostream& lexOut = lexOutStr;
#else
ostream& lexOut = cout;
#endif
#endif

SymTabMgr stm;
int debugLevel=0;
string outputFile;
const char* specFile = "";
string cppShellCmd;

bool genCCode, stopWithTypeChk;
const char* outFileSuffix = ".C";

void
printUsage(const char* cmd) {
  cerr << "Usage: " << cmd << " [-v <num>] "
    "[-o <output file>] <input file>\n" 
    "\t-v: debugging\n"
    "\t\t0: none\n"
    "\t\t1: print AST 'out' file\n"
    "\t\t2: print lots of internal debugging info\n"
    "\t-o output file name\n"
	"\t-t: Specify type of output\n"
	"\t\tc: Output C++ code (default)\n"
	"\t\tt: Stop after type-checking.\n"
	"Environment variable CPP can be used to override the default command\n"
    "for running cpp program. This variable can include the program name\n"
    "as well as command-line arguments to the command.\n";
}

int 
parseOptions(int argc, char* argv[]) {
  int c;
  opterr = 0; // Suppress printing of errors by getopt.
  int num_t_options=0;

  if (getenv(CPP_PROG_NAME) == NULL)
	cppShellCmd = DEFAULT_CPP_PROG_NAME;
  else cppShellCmd = getenv(CPP_PROG_NAME);
  cppShellCmd += " -C -traditional-cpp "; 
  // Comments should not stripped by cpp; it should also preserve white space

  while (1) {
    c = getopt(argc, argv, "-hv:t:o:");
    if (c == -1)
      break;
    
    switch(c) {
    case 1: // not a command-line option: must be the name of spec file
      if (*specFile == '\0')
		specFile = optarg;
	  else {
		cerr << "Please specify only a single input file\n";
		return -1;
	  }
      break;

    case 'v':
	  debugLevel = -1;
	  if (optarg != NULL)
		debugLevel = atol(optarg);
	  if (debugLevel < 0) {
		cerr << "Invalid debug level, ignored\n";
		debugLevel = 0;
	  }
	  else if (debugLevel > MAX_DEBUG_LEVEL) {
		cerr << "Using maximum debug level of " << MAX_DEBUG_LEVEL << "\n";
		debugLevel = MAX_DEBUG_LEVEL;
	  }
      break;

    case 'o':
      if (outputFile == "")
		outputFile = optarg;
	  else {
		cerr << "Please specify only a single output file\n";
		return -1;
	  }
      break;

    case 't':
	  if ((num_t_options++ == 0) && (optarg != NULL)) {
		  switch (*optarg) {
		  case 'c': 
			genCCode = true;
			outFileSuffix = ".C";
			break;
		  case 't': 
			stopWithTypeChk = true;
			outFileSuffix = ".C";
			break;
		  default:
			cerr << "Invalid type specified with -t option\n";
			return -1;
		  }
	  }
	  else {
		  cerr << "Use only one -t option, and provide a valid target type\n";
		  return -1;
	  }
      break;

	case 'h':
	  printUsage(argv[0]);
	  exit(1);
	  break;

    default:
	  cerr << "Invalid option\n";
      return -1;
    }
  }

  if (!genCCode)
     genCCode = !stopWithTypeChk;

  if (*specFile == '\0') 
    return -1;

  if (outputFile == "") {
	size_t pos;
    outputFile = specFile;

	if ((pos = outputFile.rfind('.')) != outputFile.npos)
	  outputFile.replace(pos, outputFile.size(), outFileSuffix);
	else outputFile += outFileSuffix;
  }

  return 0;
}

int 
main(int argc, char *argv[], char *envp[]) {

  string ccCmd;
  int optionsOK = parseOptions(argc, argv);

  if (optionsOK < 0) {
      printUsage(argv[0]);
      return 1;
  };

  cppShellCmd += specFile;
  cppShellCmd += " ";

  //cerr << "running " << cppShellCmd.c_str() << endl;
  if ((yyin = popen(cppShellCmd.c_str(), "r")) == NULL) {
    cerr << "Unexpected error in reading input file\n";
    return 1;
  }

#ifdef DEBUG_LEXER
  while (yylex() != 0);
  return 0;
#endif 

  if (yyparse() != 0)
	errMsg("Unrecoverable error at top level");
  pclose(yyin);

  GlobalEntry *ge = (GlobalEntry*)(stm.currentScope(GLOBAL_KIND));
  set<const SymTabEntry*> avlParams;
  map<VarKind, bool> valVtypes;
  if (ge != NULL) {
    if (debugLevel >= 1) {
       cout << "Finished parsing, here is the AST\n";
       ge->print(cout, 0);
    }
    if (errCount() == 0) {
       ge->typeCheck(avlParams, valVtypes);
    }
    if (errCount() == 0 && genCCode) {
       ofstream of(outputFile);//, ofstream::out);
       CGFlags cgf("");
       ge->codeGen(of, 0, cgf);
       prtMatchStats(of, "Final");
    }  
    return (errCount() > 0);
  }
  else {
     cerr << "Unexpected internal error: Could not find global symtab\n";
     return 1;
  }
}
